import warnings

import numpy as np
from keras import Sequential
from keras.layers import Embedding, LSTM, Dropout, Dense
from keras.utils import to_categorical
from keras_preprocessing.sequence import pad_sequences
from keras_preprocessing.text import Tokenizer

from KoozDawa.dawa.loader import load_kawa, clean_text
from KoozDawa.dawa.tokens import get_sequence_of_tokens

warnings.filterwarnings("ignore")
warnings.simplefilter(action='ignore', category=FutureWarning)


# 3.3 Padding the Sequences and obtain Variables : Predictors and Target
def generate_padded_sequences(input_sequences, total_words):
    max_sequence_len = max([len(x) for x in input_sequences])
    input_sequences = np.array(pad_sequences(input_sequences, maxlen=max_sequence_len, padding='pre'))

    print("Max len:", max_sequence_len)
    predictors, label = input_sequences[:, :-1], input_sequences[:, -1]
    label = to_categorical(label, num_classes=total_words)
    return predictors, label, max_sequence_len


def create_model(max_sequence_len, total_words, layers=100, dropout=0.1):  # TODO finetune
    input_len = max_sequence_len - 1
    model = Sequential()

    # Add Input Embedding Layer
    model.add(Embedding(total_words, 10, input_length=input_len))

    # Add Hidden Layer 1 - LSTM Layer
    model.add(LSTM(layers))
    model.add(Dropout(dropout))

    # Add Output Layer
    model.add(Dense(total_words, activation='softmax'))

    model.compile(loss='categorical_crossentropy', optimizer='adam')

    return model


def generate_text(model, tokenizer, seed_text="", nb_words=5, max_sequence_len=0):
    for _ in range(nb_words):
        token_list = tokenizer.texts_to_sequences([seed_text])[0]
        token_list = pad_sequences([token_list], maxlen=max_sequence_len - 1, padding='pre')
        predicted = model.predict_classes(token_list, verbose=2)

        output_word = ""
        for word, index in tokenizer.word_index.items():
            if index == predicted:
                output_word = word
                break
        seed_text += " " + output_word
    return seed_text.capitalize()


def main():
    should_train = True
    nb_epoch = 100
    max_sequence_len = 61  # TODO: Test different default
    # model_file = "../models/dawa_lstm_%i.hd5" % nb_epoch
    tokenizer = Tokenizer()

    if should_train:
        lines = load_kawa()

        corpus = [clean_text(x) for x in lines]
        print("Corpus:", corpus[:2])

        inp_sequences, total_words = get_sequence_of_tokens(corpus, tokenizer)
        predictors, label, max_sequence_len = generate_padded_sequences(inp_sequences, total_words)
        model = create_model(max_sequence_len, total_words)
        model.summary()

        model.fit(predictors, label, epochs=nb_epoch, verbose=5)
        # model.save(model_file)
    # else: # FIXME: Load and predict
    #     model = load_model(model_file)

    for sample in ["",
                   "L'étoile du sol",
                   "Elle me l'a toujours dit",
                   "Les punchlines sont pour ceux"]:
        nb_words = 200
        print(generate_text(model, tokenizer, sample, nb_words, max_sequence_len))

    with open("../output/lstm.txt", "a") as f:
        while True:
            input_text = input("> ")
            text = generate_text(model, tokenizer, input_text, nb_words, max_sequence_len)

            print(text)
            f.writelines(text)


if __name__ == '__main__':
    main()
